package com.I2CDemo;


import android.app.Activity;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.I2CDemo.R.drawable;

interface bOption
{
    public static final byte START_BIT = 0x01;
    public static final byte STOP_BIT = 0x02;
    public static final byte NACK_LAST_BYTE = 0x04; 
    public static final byte NO_DEVICE_ADDRESS = 0x08;       
}


public class I2CDemoActivity extends Activity {
	
	/*declare a CH9343 I2C interface variable*/
    public WCHI2CInterface i2cInterface;
    /*graphical objects*/
    EditText readText;
    EditText writeText;
    EditText deviceAddressText,writeDeviceAddressText;
    EditText addressText,writeAddressText;
    EditText numBytesText, writeNumBytesText;
    EditText freqText;
    TextView freqText1;
    TextView freqText2;
    TextView freqText3;
    TextView freqText4;
    //AutoCompleteTextView freqText;
    EditText statusText,writeStatusText;
    
    
    Button readButton, writeButton;
    Button freqButton;
    Button resetButton;
    
    /*local variables*/
    /*write buffer*/
    byte [] writeBuffer;
    /*read buffer*/
    byte[] readBuffer;    
    byte [] actualNumBytes;
    
    /*could have done better to parse the
     * strings
     * */
    byte [] tempBytes;
    byte tempByte;
    String tempString;
    byte tempCount=0;
    byte byteCount = 0;
    
    byte numBytes;
    byte deviceAddress;
    byte address;
    byte count;
    byte status;
    
    byte freq_tmp;
  
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        /*create editable text objects*/
        readText = (EditText)findViewById(R.id.ReadValues);
        readText.setInputType(0);
        writeText = (EditText)findViewById(R.id.WriteValues);
        freqText = (EditText)findViewById(R.id.FrequencyValue);
        freqText.setInputType(0);
        /*dirty way of displaying options*/
        freqText1 = (TextView)findViewById(R.id.FrequencyValue1);
        freqText2 = (TextView)findViewById(R.id.FrequencyValue2);
        freqText3 = (TextView)findViewById(R.id.FrequencyValue3);
        freqText4 = (TextView)findViewById(R.id.FrequencyValue4);
        
        deviceAddressText = (EditText)findViewById(R.id.DeviceAddressValue);
        writeDeviceAddressText = (EditText)findViewById(R.id.WriteDeviceAddressValue);
        
        addressText = (EditText)findViewById(R.id.MemoryAddressValue);
        writeAddressText = (EditText)findViewById(R.id.WriteMemoryAddressValue);
        
        numBytesText = (EditText)findViewById(R.id.NumberOfBytesValue);
        writeNumBytesText = (EditText)findViewById(R.id.WriteNumberOfBytesValue);
        
        statusText=(EditText)findViewById(R.id.StatusValues);
        statusText.setInputType(0);
        writeStatusText = (EditText)findViewById(R.id.WriteStatusValues);
        writeStatusText.setInputType(0);
        
        readButton = (Button)findViewById(R.id.ReadButton);
        writeButton = (Button)findViewById(R.id.WriteButton);
        freqButton = (Button)findViewById(R.id.FrequencyButton);
        resetButton = (Button)findViewById(R.id.resetButton);
        
        
        /*allocate buffer*/
        writeBuffer = new byte[255];
        readBuffer = new byte[255];
        actualNumBytes = new byte[1];
        tempBytes = new byte [3];
             
        i2cInterface = new WCHI2CInterface(this);
        resetCH9343();
       
        /*handle the reset button*/	
        resetButton.setOnClickListener(new View.OnClickListener() {
			
			//@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				//resetButton.setBackgroundResource(drawable.start);
				resetCH9343();

			}
		});

        
        /*read button*/
        readButton.setOnClickListener(new View.OnClickListener() {
			
			//@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				//readButton.setBackgroundResource(drawable.start);
				
				int numBytes_int = 0;
				int count_int = 0;
				
				/*for every read, clear the previous read*/
				readText.setText("");
				
				status = 0x00;
				/*do the sanity checks*/
				if(deviceAddressText.length() == 0){
					status = 0x4;
				}
				
				/*address to read from*/			
				if(addressText.length() == 0){
					status |= 0x04;
				}

				/*if all good, then only proceed to read*/
				if(status == 0x00)
				{		
					deviceAddress = (byte)Integer.parseInt(deviceAddressText.getText().toString(), 16);
					numBytes_int = Integer.parseInt(numBytesText.getText().toString());
					address = (byte)Integer.parseInt(addressText.getText().toString(), 16);
				
					/*blocking call*/
					byte transferOptions;
				
					byteCount = 0;
				
					/*read the bytes from the text box*/					
					transferOptions = (bOption.START_BIT | bOption.STOP_BIT | 
									   bOption.NACK_LAST_BYTE);
					status = i2cInterface.ReadData(deviceAddress, transferOptions, address,
												   numBytes_int, readBuffer, actualNumBytes);
					
					char [] displayReadbuffer;
					displayReadbuffer = new char[255];
					int displayActualNumBytes;
					displayActualNumBytes = (int)(actualNumBytes[0] & 0xff);
					
					for(count_int = 0; count_int < displayActualNumBytes; count_int++) {
						
						displayReadbuffer[count_int] = (char)readBuffer[count_int];
					}
					readText.append(String.copyValueOf(displayReadbuffer, 0, displayActualNumBytes));
				}	
				
				statusText.setText("0x"+Integer.toHexString(status));
			}
		});
        
        /*handle write click*/
        writeButton.setOnClickListener(new View.OnClickListener() {
			
			//@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				//writeButton.setBackgroundResource(drawable.start);
				
				int numBytes_int = 0;
				int byteCount_int = 0;
				int count_int = 0;
				
				/*by default, status is good*/
				status = 0x00;
				/*do the sanity checks for required values*/
				if(writeDeviceAddressText.length() == 0){
					status = 0x4;
				}
				/*the write values have to be specified*/
				if(writeText.length() == 0){
					status |= 1; /*general error*/
					
				}
				/*also need the address to write to*/
				if(writeAddressText.length() == 0){
					status |= 0x04;
				}

				if(status ==  0x00)
				{
					/*read the bytes from the write box*/
					deviceAddress = (byte)Integer.parseInt(writeDeviceAddressText.getText().toString(), 16);
					numBytes_int = writeText.length();
					address = (byte)Integer.parseInt(writeAddressText.getText().toString(), 16);
	
					/*parse the string*/
					byteCount_int = 0;
					tempCount = 0;
					/* insert 1 byte address at the start,  add 1 address byte*/
										
					byteCount_int += writeText.length();
					
					for(count_int = 0;count_int < numBytes_int;count_int++){
						writeBuffer[count_int] = (byte)writeText.getText().charAt(count_int);
					}
					
					/* subtract 1 for the address byte */
					writeNumBytesText.setText(Integer.toString(byteCount_int));
					/*blocking call*/
					byte transferOptions;
					transferOptions = (bOption.START_BIT | bOption.STOP_BIT);
					status = i2cInterface.WriteData(deviceAddress, transferOptions, address, 
							                        byteCount_int, writeBuffer, actualNumBytes);
				}
				writeStatusText.setText("0x"+Integer.toHexString(status)); 
				
			}
		});
        
        /*handle write click*/
        freqButton.setOnClickListener(new View.OnClickListener() {
			
			//@Override
			public void onClick(View v) {
				
				// TODO Auto-generated method stub
				freqText1.setVisibility(View.INVISIBLE);
				freqText2.setVisibility(View.INVISIBLE);
				freqText3.setVisibility(View.INVISIBLE);
				freqText4.setVisibility(View.INVISIBLE);
				
				
				/*read the bytes from the write box*/
				if(freqText.length() != 0)
					freq_tmp = (byte) Integer.parseInt(freqText.getText().toString());
				else
					freq_tmp = 92;
				
				freqText.setText(Integer.toString(freq_tmp));
				i2cInterface.SetFrequency(freq_tmp);
			}
		});

        
        /*select the frequency*/
        freqText.setOnTouchListener(new View.OnTouchListener() {
			
			//@Override
			public boolean onTouch(View v, MotionEvent event) {
				// TODO Auto-generated method stub
				freqText1.setVisibility(View.VISIBLE);
				freqText2.setVisibility(View.VISIBLE);
				freqText3.setVisibility(View.VISIBLE);
				freqText4.setVisibility(View.VISIBLE);
				
				return false;
			}
		});
        
        
        /*set the selected value*/
        freqText1.setOnTouchListener(new View.OnTouchListener() {
			
			//@Override
			public boolean onTouch(View v, MotionEvent event) {
				// TODO Auto-generated method stub
				freqText.setText(freqText1.getText().toString());
				freqText1.setVisibility(View.INVISIBLE);
				freqText2.setVisibility(View.INVISIBLE);
				freqText3.setVisibility(View.INVISIBLE);
				freqText4.setVisibility(View.INVISIBLE);
				return false;
			}
		});

        /*set the selected value*/
        freqText2.setOnTouchListener(new View.OnTouchListener() {
			
			//@Override
			public boolean onTouch(View v, MotionEvent event) {
				// TODO Auto-generated method stub
				freqText.setText(freqText2.getText().toString());
				freqText1.setVisibility(View.INVISIBLE);
				freqText2.setVisibility(View.INVISIBLE);
				freqText3.setVisibility(View.INVISIBLE);
				freqText4.setVisibility(View.INVISIBLE);
				return false;
			}
		});
        
        /*set the selected value*/
        freqText3.setOnTouchListener(new View.OnTouchListener() {
			
			//@Override
			public boolean onTouch(View v, MotionEvent event) {
				// TODO Auto-generated method stub
				freqText.setText(freqText3.getText().toString());
				freqText1.setVisibility(View.INVISIBLE);
				freqText2.setVisibility(View.INVISIBLE);
				freqText3.setVisibility(View.INVISIBLE);
				freqText4.setVisibility(View.INVISIBLE);
				return false;
			}
		});
        
        /*set the selected value*/
        freqText4.setOnTouchListener(new View.OnTouchListener() {
			
			//@Override
			public boolean onTouch(View v, MotionEvent event) {
				// TODO Auto-generated method stub
				freqText.setText(freqText4.getText().toString());
				freqText1.setVisibility(View.INVISIBLE);
				freqText2.setVisibility(View.INVISIBLE);
				freqText3.setVisibility(View.INVISIBLE);
				freqText4.setVisibility(View.INVISIBLE);
				return false;
			}
		});
        
        
    }
    
    protected void resetCH9343()
    {
		freqText.setText("92");		
		readText.setText("");		
	    deviceAddressText.setText("");
	    addressText.setText("");
	    numBytesText.setText("");
	    statusText.setText("");	    
	    writeText.setText("");
	    writeDeviceAddressText.setText("");
	    writeAddressText.setText("");	    
	    writeNumBytesText.setText("");
	    writeStatusText.setText("");	    
		i2cInterface.Reset();    
    }
    
    
    @Override
    protected void onResume() {
        // Ideally should implement onResume() and onPause()
        // to take appropriate action when the activity looses focus
        super.onResume();
        i2cInterface.ResumeAccessory();
    }

    @Override
    protected void onPause() {
        // Ideally should implement onResume() and onPause()
        // to take appropriate action when the activity looses focus
        super.onPause();
    }
    
    @Override 
    protected void onDestroy(){
    	
    	i2cInterface.DestroyAccessory();
    	super.onDestroy();
    }
}