//User must modify the below package with their package name
package com.I2CDemo; 
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbAccessory;
import android.hardware.usb.UsbManager;
import android.os.Handler;
import android.os.Message;
import android.os.ParcelFileDescriptor;
import android.util.Log;
import android.widget.Toast;


/******************************CH9343 I2C interface class******************************************/
public class WCHI2CInterface extends Activity
{
	//define Endpoint 'vMaxPacketSize',depending on CH9343 USB host
	private static final byte  IN_EP = 64;
	private static final byte OUT_EP = 64;
	
	private byte IN_maxnumbytes = (IN_EP - 3); /* maximum data bytes, except command, cmd:status:len:data */
	private byte OUT_maxnumbytes = (OUT_EP - 3);
	
	public boolean senddata_first_flag = true;
	public boolean recvdata_first_flag = true;
	
	private static final String ACTION_USB_PERMISSION =    "com.I2CDemo.USB_PERMISSION";
	public UsbManager usbmanager;
	public UsbAccessory usbaccessory;
	public PendingIntent mPermissionIntent;
	public ParcelFileDescriptor filedescriptor;
	public FileInputStream inputstream;
	public FileOutputStream outputstream;
	public boolean mPermissionRequestPending = false;
	public boolean READ_ENABLE = true;
	public boolean accessory_attached = false;
	public handler_thread handlerThread;
	
	private byte [] usbdata; 
    private byte []	writeusbdata;
    private int readcount;
    private byte status;
    private byte  maxnumbytes = 59;
    private int maxnumbytes_int = 255;
    public boolean datareceived = true;
	
    
    public Context global_context;
   
    public static String ManufacturerString = "mManufacturer=WCH";
    public static String ModelString = "mModel=WCHI2CDemo";
    public static String VersionString = "mVersion=1.0";	
		
		/*constructor*/
	 public WCHI2CInterface(Context context){
		 	super();
		 	global_context = context;
			/*shall we start a thread here or what*/
			usbdata = new byte[64]; 
	        writeusbdata = new byte[64];
	        
	        /***********************USB handling******************************************/
			
	        usbmanager = (UsbManager) context.getSystemService(Context.USB_SERVICE);
	       // Log.d("LED", "usbmanager" +usbmanager);
	        mPermissionIntent = PendingIntent.getBroadcast(context, 0, new Intent(ACTION_USB_PERMISSION), 0);
	        IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
	       filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);
	       context.registerReceiver(mUsbReceiver, filter);
	        
	       inputstream = null;
	       outputstream = null;
		}
	 
	 	/*reset method*/
	 	public void Reset()
	 	{
	 		/*create the packet*/
	 		writeusbdata[0] = 0x43;
			writeusbdata[1] = 0x00;
			writeusbdata[2] = 0x00;
			writeusbdata[3] = 0x00;
			writeusbdata[4] = 0x00;
			
			/*send the packet over the USB*/
			SendPacket(5);
	 	}
	 	
	 	
	 	/*SetFrequency*/
	 	public void SetFrequency(byte freq)
	 	{
	 		/*check for maximum and minimum freq*/
	 		if(freq > 92)
	 			freq = 92;
	 		
	 		if (freq < 23)
	 			freq = 23;
	 		
	 		/*create the packet*/
	 		writeusbdata[0] = 0x40;
	 		switch(freq)
	 		{
	 		case 23:
	 			writeusbdata[1] = 0x00;
	 			break;
	 		case 44:
	 			writeusbdata[1] = 0x01;
	 			break;
	 		case 60:
	 			writeusbdata[1] = 0x02;
	 			break;
	 		default:
	 			writeusbdata[1] = 0x03;
	 			break;
	 		}
			writeusbdata[2] = 0x00;
			writeusbdata[3] = 0x00;
			writeusbdata[4] = 0x00;
			
			/*send the packet over the USB*/
			SendPacket(5);
	 	}
	 	
	 	
	 	/*write data*/
	 	public byte WriteData(byte i2cDeviceAddress,byte transferOptions,
	 						 byte i2cDataAddress,
	 					     int numBytes, byte[] buffer, 
	 					     byte [] actualNumBytes)
	 	{
	 		int have_send = 0;
	 		
	 		status = 0x01; /*error by default*/
	 		/*
	 		 * if num bytes are more than maximum limit
	 		 */
	 		if(numBytes < 1){
	 			
	 			actualNumBytes[0] = (byte)0x00;
	 			/*return the status with the error in the command*/
	 			return status;
	 		}
	 		
	 		/*check for maximum limit*/
	 		if(numBytes > maxnumbytes_int){
	 			numBytes = maxnumbytes_int;
	 		}
	 		
	 		/*prepare the usbpacket*/
	 		writeusbdata[0] = 0x41;
	 		writeusbdata[1] = transferOptions;
	 		writeusbdata[2] = i2cDeviceAddress;
	 		writeusbdata[3] = i2cDataAddress;
	 		writeusbdata[4] = (byte)(numBytes & 0xff);
	 		
	 		if( (numBytes + 5) > 64 )
	 		{
	 			int retval = (numBytes +5)/64;
	 			
	 			for( int i = 0; i < retval; i++){
	 				if( i == 0){
	 					for(int count_x = 0; count_x < (64-5); count_x++){
	 						writeusbdata[5+count_x] = (byte)buffer[have_send++];
	 					}
	 				}
	 				else
	 				{
	 					for( int count_y = 0; count_y < 64; count_y++) {
	 						writeusbdata[count_y] = (byte)buffer[have_send++];
	 					}
	 				}
	 				
	 				SendPacket((int)64);
	 			}
	 			
	 			if( (numBytes + 5) - (retval*64) > 0) {
	 				for(int count_h = 0; count_h < ( (numBytes + 5) - (retval*64)); count_h++) {
	 					writeusbdata[count_h] = (byte)buffer[have_send++];
	 				}
	 				
	 				SendPacket((int)((numBytes + 5) - (retval*64)));
	 			}
	 		}
	 		else if( (numBytes + 5) <= 64 )
	 		{
	 			/*prepare the packet to be sent*/
		 		for(int count_z = 0;count_z < numBytes;count_z++){
		 			writeusbdata[5+count_z] = (byte)buffer[count_z];
		 		}
		 		
		 		SendPacket((int)(numBytes+5));
	 		}
	 	
	 		
	 		datareceived = false;
	 		/*wait while the data is received*/
	 		/*FIXME, may be create a thread to wait on , but any
	 		 * way has to wait in while loop
	 		 */
	 		while(true){
	 			
	 			if(datareceived == true){
	 				break;
	 			}
	 		}
	 		
	 		/*by default it error*/
	 		status = 0x01;
	 		/*check for the received data*/
	 		if(usbdata[0] == 0x41)
	 		{
	 			/*check for return error status*/
	 			status = usbdata[1];
	 			
	 			/*updated the written bytes*/
	 			actualNumBytes[0] = usbdata[2];
	 		}
	 		//datareceived = false;
	 		return status;
	 	}
	 	
	 	/*read data*/
	 	public byte ReadData(byte i2cDeviceAddress,byte transferOptions,
	 						  byte i2cDataAddress,
	 						  int numBytes, byte[] readBuffer,
	 						  byte [] actualNumBytes)
	 	{
	 			int total_readcount = 0; //actually length readed
	 			int count_read = 0;
	 			int readable_numBytes = 0;
	 		
	 			status = 0x01; /*error by default*/
	 			
	 			/*should be at least one byte to read*/
	 			if(numBytes < 1){
	 				return status;
	 			}
	 			
	 			/*check for max limit*/
	 			if(numBytes > maxnumbytes_int){
	 				numBytes = maxnumbytes_int;
	 			}
	 			
	 			
	 			/*prepare the packet to send this command*/
	 			writeusbdata[0] = 0x42; /*read data command*/
	 			writeusbdata[1] = transferOptions;  /*transfer options*/
	 			writeusbdata[2] = i2cDeviceAddress;  /*device address*/
	 			writeusbdata[3] = i2cDataAddress; /*Data address*/
	 			writeusbdata[4] = (byte)(numBytes & 0xff); /*number of bytes*/
	 			
	 			/*send the data on USB bus*/
	 			SendPacket((int)5);
	 			
	 			
	 			while(true)
	 			{
	 				receive_reply();
	 				if( recvdata_first_flag == true )
	 				{
	 					if( usbdata[0] == 0x42 ) {
	 						status = usbdata[1];
	 						readable_numBytes = (int)(usbdata[2] & 0xff);
	 						if( readable_numBytes < numBytes ) {
	 							numBytes = readable_numBytes;
	 						}
	 					}
	 					else {
	 						Toast.makeText(global_context, "Read Error", Toast.LENGTH_SHORT).show();
	 						break;
	 					}
	 					
	 					recvdata_first_flag = false;
	 					total_readcount += (readcount - 3);
	 					for (int count_x = 0; count_x < (readcount - 3); count_x++) {
	 						readBuffer[count_x] = usbdata[count_x + 3];
	 					}
	 					
	 					if( (total_readcount == numBytes) || (total_readcount < OUT_maxnumbytes) ) {
	 						break;
	 					}
	 					
	 					continue;
	 				}
	 				
	 				if( readcount == 0 ){
	 					break;
	 				}
	 				
	 				for (int count_y = 0; count_y < readcount; count_y++ ) {
	 					readBuffer[count_y + total_readcount] = usbdata[count_y];
	 				}
	 				total_readcount += (byte)(readcount);
	 				
	 				if( (total_readcount == numBytes) || readcount < (OUT_maxnumbytes + 3) ){
	 					break;
	 				}
	 					
	 			}
	 			
	 			//Toast.makeText(global_context, "Read Finshed:"+, Toast.LENGTH_SHORT).show();
	 			actualNumBytes[0] = (byte)(total_readcount & 0xff);
	 			recvdata_first_flag = true;
	 			
	 		return status;
	 	}
	 	
	 	
	 	private void receive_reply()
		{
			datareceived = false;
			while(true) 
			{
				if (datareceived == true) {
					break;
				}
			}
		}
	 		
		
		/*method to send on USB*/
		private void SendPacket(int numBytes)
		{
			try {
				if(outputstream != null){
					outputstream.write(writeusbdata, 0,numBytes);
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		
		
		/*resume accessory*/
		public void ResumeAccessory()
		{
			// Intent intent = getIntent();
			if (inputstream != null && outputstream != null) {
				return;
			}
			
			UsbAccessory[] accessories = usbmanager.getAccessoryList();
			if(accessories != null)
			{
				Toast.makeText(global_context, "Accessory Attached", Toast.LENGTH_SHORT).show();
			}
			else
			{
				accessory_attached = false;
				return;
			}

			UsbAccessory accessory = (accessories == null ? null : accessories[0]);
			if (accessory != null) {
				if( -1 == accessory.toString().indexOf(ManufacturerString))
				{
					Toast.makeText(global_context, "Manufacturer is not matched!", Toast.LENGTH_SHORT).show();
					return;
				}

				if( -1 == accessory.toString().indexOf(ModelString))
				{
					Toast.makeText(global_context, "Model is not matched!", Toast.LENGTH_SHORT).show();
					return;
				}
				
				if( -1 == accessory.toString().indexOf(VersionString))
				{
					Toast.makeText(global_context, "Version is not matched!", Toast.LENGTH_SHORT).show();
					return;
				}
				
				Toast.makeText(global_context, "Manufacturer, Model & Version are matched!", Toast.LENGTH_SHORT).show();
				accessory_attached = true;

				if (usbmanager.hasPermission(accessory)) {
					OpenAccessory(accessory);
				} 
				else
				{
					synchronized (mUsbReceiver) {
						if (!mPermissionRequestPending) {
							Toast.makeText(global_context, "Request USB Permission", Toast.LENGTH_SHORT).show();
							usbmanager.requestPermission(accessory,
									mPermissionIntent);
							mPermissionRequestPending = true;
						}
				}
			}
			} else {}

		}
		
		/*destroy accessory*/
		public void DestroyAccessory(){
			global_context.unregisterReceiver(mUsbReceiver);
			if(accessory_attached == true)
			{
				READ_ENABLE = false;
				byte i2cDeviceAddress = 1;
				byte transferOptions = bOption.START_BIT;
				byte i2cDataAddress = 1;
			    int  numBytes = 2;
			    byte [] actualNumBytes = new byte[1];
			    byte [] readBuffer = new byte[60];
			    readBuffer[0] = 1;

			    //FIXME
				ReadData(i2cDeviceAddress,transferOptions,
						 i2cDataAddress,
						  numBytes, readBuffer,
						  actualNumBytes);
			}
			CloseAccessory();
		}
		

		
/*********************helper routines*************************************************/		
		
		public void OpenAccessory(UsbAccessory accessory)
		{
			filedescriptor = usbmanager.openAccessory(accessory);
			if(filedescriptor != null){
				usbaccessory = accessory;
				FileDescriptor fd = filedescriptor.getFileDescriptor();
				inputstream = new FileInputStream(fd);
				outputstream = new FileOutputStream(fd);
				/*check if any of them are null*/
				if(inputstream == null || outputstream==null){
					return;
				}
			}
			
			handlerThread = new handler_thread(inputstream);
			handlerThread.start();
		}
		
		private void CloseAccessory()
		{
			try{
				if(filedescriptor != null)
					filedescriptor.close();
				
			}catch (IOException e){}
			
			try {
				if(inputstream != null)
						inputstream.close();
			} catch(IOException e){}
			
			try {
				if(outputstream != null)
						outputstream.close();
				
			}catch(IOException e){}
			/*FIXME, add the notfication also to close the application*/
			
			filedescriptor = null;
			inputstream = null;
			outputstream = null;
		
			System.exit(0);
		}
		
				
		/***********USB broadcast receiver*******************************************/
	    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() 
		{
			@Override
			public void onReceive(Context context, Intent intent) 
			{
				String action = intent.getAction();
				if (ACTION_USB_PERMISSION.equals(action)) 
				{
					synchronized (this)
					{
						UsbAccessory accessory = (UsbAccessory) intent.getParcelableExtra(UsbManager.EXTRA_ACCESSORY);
						if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false))
						{
							Toast.makeText(global_context, "Allow USB Permission", Toast.LENGTH_SHORT).show();
							OpenAccessory(accessory);
						} 
						else 
						{
							Toast.makeText(global_context, "Deny USB Permission", Toast.LENGTH_SHORT).show();
							Log.d("LED", "permission denied for accessory "+ accessory);
							
						}
						mPermissionRequestPending = false;
					}
				} 
				else if (UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) 
				{
						CloseAccessory();
				}else
				{
					Log.d("LED", "....");
				}
			}	
		};
	    
		
		/*usb input data handler*/
		private class handler_thread  extends Thread {
			FileInputStream instream;
			
			handler_thread(FileInputStream stream ){
				instream = stream;
			}
			
			public void run()
			{
				
				while(READ_ENABLE == true)
				{
					try
					{
						/*dont overwrite the previous buffer*/
						if((instream != null) && (datareceived==false))
						{
							readcount = instream.read(usbdata,0,64);
							if(readcount > 0)
							{
								datareceived = true;
								/*send only when you find something*/	
							}
						}
					}catch (IOException e){}
				}
			}
		}
	    
		
		
	}